"""
Created by Prof. Xin Yuan (Department of Computer Scinece,
Florida State University) for COP4521
This program computes the smallest prime triplets where the values of the
primes are no less than N (a value input by the user).

Timing results in linprog:

<linprog3:591> python3 assignment3_provided.py
Input n: 1000000000000
The smallest triplet larger than 1000000000000 is (1000000005073, 1000000005077, 1000000005079)

Time to build the list of prime numbers: 2.017319917678833 seconds
Time to search for prime triplet: 2.96591854095459 seconds
Total time: 4.983248233795166 seconds
<linprog3:592> python3 assignment3_provided.py
Input n: 5000000000000
The smallest triplet larger than 5000000000000 is (5000000002027, 5000000002031, 5000000002033)

Time to build the list of prime numbers: 5.645062685012817 seconds
Time to search for prime triplet: 2.5009918212890625 seconds
Total time: 8.146062135696411 seconds
<linprog3:593> 
"""


import math
import time

primes = [2, 3, 5]

def makePrimeList(n):
    for i in range(6, n):
        if (isPrime(i)):
            primes.append(i)
            
def isPrime(n):
    """
    This function assumes that primes is sorted and includes
    all prime numbers up to at least int(math.sqrt(n)) + 1
    """
    
    i=0
    b = int(math.sqrt(n)) + 1
    pLen = len(primes)
    while i < pLen and (primes[i] < b):
        if n % primes[i] == 0:
            return False
        i = i + 1
    return True

n = int(input("Input n: "))
startT = time.time()


# assume the primes in the prime triplet are
# not larger than n + 100000
largestPrimeNeeded = int (math.sqrt(n + 100000))

if (n <= 5):
    msg = 'The smallest triplet larger than ' + str(n) + ' is ('
    msg = msg + '5, 7, 11)'
    print(msg)
    exit()

listStartT = time.time()
makePrimeList(largestPrimeNeeded)
listStopT = time.time()

searchStartT = time.time()

m1 = 2
m2 = 2
m3 = 2
cur = n
while True:
    if isPrime(cur):
        m1,m2,m3 = m2, m3, cur
        if (m3 - m1 == 6) :
            msg = 'The smallest triplet larger than ' + str(n) + ' is ('
            msg = msg + str(m1) + ', ' + str(m2)
            msg = msg + ', ' + str(m3) + ')'
                
            print(msg)
            break;
    cur = cur + 1
stopT = time.time()

print()
print(f'Time to build the list of prime numbers: {listStopT - listStartT} seconds')
print(f'Time to search for prime triplet: {stopT - searchStartT} seconds')
print(f'Total time: {stopT - startT} seconds')

